from pathlib import Path

from musurgia.fractaltree.fractaltree import FractalTree
from musurgia.pdf.line import HorizontalLineSegment
from musurgia.pdf.pdf import Pdf
from musurgia.pdf.rowcolumn import DrawObjectColumn, DrawObjectRow
from musurgia.unittest import TestCase

path = Path(__file__)


def fill_tree_with_columns(fractal_tree, factor=1):
    max_large_ml = 6
    min_large_ml = 3
    for node in fractal_tree.traverse():
        node.manual_graphic = DrawObjectColumn()
        segment = node.manual_graphic.add_draw_object(HorizontalLineSegment(length=node.value * factor))
        ml_length = (max_large_ml - (
                node.get_distance() * (max_large_ml - min_large_ml) / fractal_tree.number_of_layers)) / 2
        segment.start_mark_line.length = ml_length
        segment.end_mark_line.length = ml_length
        if node.get_children():
            node.children_manual_graphics_row = node.manual_graphic.add_draw_object(DrawObjectRow(top_margin=3))
        if node.up:
            node.up.children_manual_graphics_row.add_draw_object(node.manual_graphic)
        if not node.up or node.up.get_children().index(node) == len(node.up.get_children()) - 1:
            node.manual_graphic.draw_objects[0].end_mark_line.show = True
            node.manual_graphic.draw_objects[0].end_mark_line.length *= 2
        if not node.up or node.up.get_children().index(node) == 0:
            node.manual_graphic.draw_objects[0].start_mark_line.length *= 2


class TestFractal(TestCase):
    def setUp(self) -> None:
        self.pdf = Pdf(orientation='l')
        self.ft = FractalTree(value=20)
        self.ft.add_layer()
        self.factor = 2
        fill_tree_with_columns(self.ft, self.factor)

    def test_draw(self):
        with self.file_path(path, 'draw', 'pdf') as pdf_path:
            self.pdf.translate_page_margins()
            self.pdf.draw_ruler('h')
            self.pdf.draw_ruler('v')
            self.pdf.translate(10, 10)
            self.ft.manual_graphic.draw(self.pdf)
            self.pdf.write(pdf_path)

    def test_draw_with_text(self):
        leaf = self.ft.get_leaves()[1]
        leaf_start_mark_line = leaf.manual_graphic.draw_objects[0].start_mark_line
        leaf_start_mark_line.add_label('bla')
        leaf_start_mark_line.add_label('blue')
        row = self.ft.manual_graphic.draw_objects[1]
        row.top_margin = max(
            [do.draw_objects[0].start_mark_line.get_above_text_labels_height() for do in row.draw_objects])
        print(row.top_margin)
        with self.file_path(path, 'draw_with_text', 'pdf') as pdf_path:
            self.pdf.translate_page_margins()
            self.pdf.draw_ruler('h')
            self.pdf.draw_ruler('v')
            self.pdf.translate(10, 10)
            self.ft.manual_graphic.draw(self.pdf)
            self.pdf.write(pdf_path)

    def test_draw_filled_tree(self):
        ft = FractalTree(value=20)
        ft.add_layer()
        ft.get_children()[1].add_layer()
        fill_tree_with_columns(ft, 2)

        with self.file_path(path, 'draw_filled_tree', 'pdf') as pdf_path:
            self.pdf.translate_page_margins()
            self.pdf.draw_ruler('h')
            self.pdf.draw_ruler('v')
            self.pdf.translate(10, 10)
            ft.manual_graphic.draw(self.pdf)
            self.pdf.write(pdf_path)

    def test_draw_unpruned_tree_with_some_text(self):
        ft = FractalTree(value=20)
        ft.add_layer()
        ft.get_children()[0].add_layer()
        ft.get_children()[-1].add_layer()
        fill_tree_with_columns(ft, 2)
        selected_node = ft.select_index([1])
        child_1 = selected_node.get_children()[1]
        child_2 = selected_node.get_children()[2]
        lb1 = child_1.manual_graphic.draw_objects[0].start_mark_line.add_label(1, font_size=8, bottom_margin=1)
        lb2 = child_1.manual_graphic.draw_objects[0].start_mark_line.add_label(2, font_size=8, bottom_margin=1)
        lb3 = child_2.manual_graphic.draw_objects[0].start_mark_line.add_label(3, font_size=8, bottom_margin=1)

        for child in ft.get_children():
            try:
                child.manual_graphic.draw_objects[1].top_margin += 2
            except IndexError:
                pass

        with self.file_path(path, 'draw_unpruned_tree_with_some_text', 'pdf') as pdf_path:
            self.pdf.translate_page_margins()
            self.pdf.draw_ruler('h')
            self.pdf.draw_ruler('v')
            self.pdf.translate(10, 10)
            ft.manual_graphic.draw(self.pdf)
            self.pdf.write(pdf_path)

    


from unittest import TestCase

from quicktions import Fraction

from musurgia.fractaltree.fractalmusic import FractalMusic


class Test(TestCase):
    def setUp(self) -> None:
        self.fm = FractalMusic(tempo=72, quarter_duration=10)

    def test_1(self):
        self.fm.change_quarter_duration(20)
        expected = 20
        self.assertEqual(expected, self.fm.quarter_duration)

    def test_2(self):
        self.fm.add_layer()
        self.fm.get_children()[0].change_quarter_duration(10)
        expected = 15
        self.assertEqual(expected, self.fm.quarter_duration)

    def test_3(self):
        self.fm.add_layer()
        self.fm.get_children()[0].change_quarter_duration(10)
        expected = [Fraction(10, 1), Fraction(5, 3), Fraction(10, 3)]
        self.assertEqual(expected, [child.quarter_duration for child in self.fm.get_children()])

    def test_4(self):
        self.fm.add_layer()
        self.fm.change_quarter_duration(15)
        expected = [Fraction(15, 2), Fraction(5, 2), Fraction(5, 1)]
        self.assertEqual(expected, [child.quarter_duration for child in self.fm.get_children()])

    def test_5(self):
        self.fm.add_layer()
        self.fm.add_layer()
        self.fm.get_children()[0].change_quarter_duration(10)
        expected = [[Fraction(15, 1)],
                    [10, Fraction(5, 3), Fraction(10, 3)],
                    [[Fraction(5, 3), Fraction(10, 3), Fraction(5, 1)],
                     [Fraction(5, 6), Fraction(5, 18), Fraction(5, 9)],
                     [Fraction(10, 9), Fraction(5, 3), Fraction(5, 9)]]]
        self.assertEqual(expected,
                         [self.fm.get_layer(layer=i, key='quarter_duration') for i in
                          range(self.fm.number_of_layers + 1)])



from musurgia.pdf.labeled import Labeled
from musurgia.pdf.newdrawobject import DrawObject
from musurgia.pdf.positioned import RelativeXNotSettableError, RelativeYNotSettableError


class MarkLine(DrawObject, Labeled):
    def __init__(self, parent, placement='start', height=3, thickness=1, show=True, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.show = show
        self.parent = parent
        self._height = None
        self._placement = None

        self.placement = placement
        self.height = height
        self.position = None
        self.thickness = thickness

    @property
    def left_margin(self):
        return 0

    @property
    def right_margin(self):
        return 0

    @property
    def top_margin(self):
        return 0

    @property
    def bottom_margin(self):
        return 0

    @property
    def relative_x(self):
        if self.placement == 'start':
            return 0
        else:
            return self.parent.actual_length

    @relative_x.setter
    def relative_x(self, val):
        if val is not None:
            raise RelativeXNotSettableError()

    @property
    def relative_y(self):
        return self.parent.relative_y - self.height / 2

    @relative_y.setter
    def relative_y(self, val):
        if val is not None:
            raise RelativeYNotSettableError()

    @property
    def placement(self):
        return self._placement

    @placement.setter
    def placement(self, val):
        permitted = ['start', 'end']
        if val not in permitted:
            raise ValueError('placement.value {} must be in {}'.format(val, permitted))
        self._placement = val

    @property
    def height(self):
        return self._height

    @height.setter
    def height(self, val):
        self._height = val

    def get_relative_x2(self):
        return self.relative_x

    def get_relative_y2(self):
        return self.relative_y + self.height

    def draw(self, pdf):
        # pdf.translate(self.relative_x, self.relative_y)
        # no margins
        x1 = self.relative_x
        x2 = x1
        y1 = self.relative_y
        y2 = y1 + self.height
        if self.show:
            for i in range(self.thickness):
                grid = 0.1
                pdf.line(x1=x1 + i * grid, y1=y1, x2=x2 + i * grid, y2=y2)
        for text_label in self.text_labels:
            text_label.draw(pdf)



from pathlib import Path

from musurgia.pdf.line import HorizontalLineSegment
from musurgia.pdf.pdf import Pdf
from musurgia.pdf.text import Text
from musurgia.unittest import TestCase

path = Path(__file__)


class TestText(TestCase):
    def setUp(self) -> None:
        self.pdf = Pdf()

    def test_draw(self):
        t = Text('The fox is going to be dead.')
        with self.file_path(path, 'draw', 'pdf') as pdf_path:
            self.pdf.translate_page_margins()
            self.pdf.draw_ruler('h')
            self.pdf.draw_ruler('v')
            self.pdf.translate(10, 10)
            t.draw(self.pdf)
            self.pdf.write(pdf_path)

    def test_draw_with_top_margin(self):
        t = Text('The fox is going to be dead.')
        t.top_margin = 5
        with self.file_path(path, 'draw_with_top_margin', 'pdf') as pdf_path:
            self.pdf.translate_page_margins()
            self.pdf.draw_ruler('h')
            self.pdf.draw_ruler('v')
            self.pdf.translate(10, 10)
            t.draw(self.pdf)
            self.pdf.write(pdf_path)

    def test_get_height(self):
        t = Text('fox is going to be dead.')
        t.font_size = 14
        t.top_margin = 3
        expected = 7.682066666666666
        actual = t.get_height()
        self.assertEqual(expected, actual)

    def test_height_graphical(self):
        t = Text('The fox is going to be dead.')
        with self.file_path(path, 'height_graphical', 'pdf') as pdf_path:
            self.pdf.translate_page_margins()
            self.pdf.draw_ruler('h')
            self.pdf.draw_ruler('v')
            self.pdf.translate(10, 10)
            hls = HorizontalLineSegment(length=t.get_text_width(), relative_y=-t.get_text_height() * 3 / 4)
            hls.start_mark_line.length = t.get_height()
            hls.end_mark_line.show = True

            with self.pdf.saved_state():
                self.pdf.rect(hls.relative_x, hls.relative_y, hls.get_width(), t.get_height())
                hls.draw(self.pdf)
            t.draw(self.pdf)

            self.pdf.write(pdf_path)

    def test_draw_multiple(self):
        t1 = Text(value='Fox is going to be dead.')
        t2 = Text(value='What should we do??', relative_y=0)
        t3 = Text(value='What should we do??', relative_y=0)
        with self.file_path(path, 'draw_multiple', 'pdf') as pdf_path:
            self.pdf.translate_page_margins()
            self.pdf.draw_ruler('h')
            self.pdf.draw_ruler('v')
            self.pdf.translate(10, 10)
            t1.draw(self.pdf)
            self.pdf.translate(0, t1.get_height())
            t2.draw(self.pdf)
            self.pdf.translate(0, t1.get_height())
            t3.draw(self.pdf)
            self.pdf.translate(0, t1.get_height())
            self.pdf.write(pdf_path)

    def test_draw_multiple_with_relative_y(self):
        t1 = Text(value='Fox is going to be dead.', relative_x=10, relative_y=10)
        t2 = Text(value='What should we do??', relative_x=10, relative_y=20)
        t3 = Text(value='What should we do??', relative_x=10, relative_y=30)

        with self.file_path(path, 'draw_multiple_with_relative_y', 'pdf') as pdf_path:
            self.pdf.translate_page_margins()
            self.pdf.draw_ruler('h')
            self.pdf.draw_ruler('v')

            t1.draw(self.pdf)
            t2.draw(self.pdf)
            t3.draw(self.pdf)

            self.pdf.write(pdf_path)



import os
from fractions import Fraction
from itertools import cycle

from musicscore.musicstream.streamvoice import SimpleFormat
from musicscore.musictree.treescoretimewise import TreeScoreTimewise

from musurgia.chordfield.chordfield import Breathe, ChordField
from musurgia.chordfield.valuegenerator import ValueGenerator

path = str(os.path.abspath(__file__).split('.')[0])
score = TreeScoreTimewise()
proportions = (0, 2, 2, 0, 0)
breakpoints = (1, Fraction(1, 4), 1)
quarter_durations = [2, 4]
breathe = Breathe(proportions=proportions, breakpoints=breakpoints, quarter_duration=sum(quarter_durations),
                  quantize=1)
###
breathe = breathe.__deepcopy__()
breathe.midi_generator = ValueGenerator(cycle([60]))
###
parent_chord_field = ChordField(duration_generator=breathe.duration_generator.__deepcopy__())
# parent_chord_field = ChordField(
#     duration_generator=ValueGenerator(iter(list(breathe.duration_generator.__deepcopy__()))))
for i in range(len(quarter_durations)):
    quarter_duration = quarter_durations[i]
    midi = 60 + i
    parent_chord_field.add_child(
        ChordField(midi_generator=ValueGenerator(cycle([midi])), long_ending_mode='self_extend',
                   short_ending_mode='self_shrink', quarter_duration=quarter_duration))
####
copy_parent_chord_field = parent_chord_field.__deepcopy__()
####

####
breathe.simple_format.to_stream_voice().add_to_score(score=score, part_number=1)
parent_chord_field.simple_format.to_stream_voice().add_to_score(score=score, part_number=2)


# print(sum([fractal_tree.quarter_duration for fractal_tree in copy_parent_chord_field.children]))
# # print(copy_parent_chord_field.children[0].simple_format.quarter_duration)
# print(([float(fractal_tree.quarter_duration) for fractal_tree in copy_parent_chord_field.children]))
# print(float(copy_parent_chord_field.children[0].simple_format.quarter_duration))
# print(([float(fractal_tree.quarter_duration) for fractal_tree in copy_parent_chord_field.children]))
# print(float(copy_parent_chord_field.children[1].simple_format.quarter_duration))
# print(([float(fractal_tree.quarter_duration) for fractal_tree in copy_parent_chord_field.children]))
def iterate(chord_field):
    while True:
        try:
            print('pos:{}, pospar:{}, genpos {}'.format(chord_field.position, chord_field.position_in_parent,
                                                        chord_field.duration_generator.position,
                                                        ))
            next = chord_field.__next__()
            print('next duration:{}'.format(next.quarter_duration))
        except StopIteration:
            break


iterate(copy_parent_chord_field.children[0])
print()
iterate(copy_parent_chord_field.children[1])
simple_format = SimpleFormat()
for child in copy_parent_chord_field.children:
    simple_format.extend(child.simple_format)
simple_format.to_stream_voice().add_to_score(score=score, part_number=3)
print(float(sum([child.quarter_duration for child in copy_parent_chord_field.children])))
##

xml_path = path + '_test_1.xml'
score.write(xml_path)



import os

from musurgia.unittest import TestCase

path = str(os.path.abspath(__file__).split('.')[0])


class Test(TestCase):
    def test_1(self):
        actual_file_path = path + '_a1.pdf'
        expected_file_path = path + '_a2.pdf'
        self.assertCompareFiles(actual_file_path, expected_file_path)

    def test_2(self):
        with self.assertRaises(AssertionError):
            actual_file_path = path + '_a1.pdf'
            expected_file_path = path + '_b.pdf'
            self.assertCompareFiles(actual_file_path, expected_file_path)



from musurgia.pdf.named import Named
from musurgia.timeline.abstractvoice import AbstractVoice


class VoiceError(Exception):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class VoiceSegment(object):
    def __init__(self, start, stop, parent, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._start = None
        self._stop = None
        self.start = start
        self.stop = stop
        self._parent = None
        self.parent = parent
        self._lines = None

    @property
    def start(self):
        return self._start

    @start.setter
    def start(self, val):
        if not isinstance(val, int):
            raise TypeError('start.value must be of type int not{}'.format(type(val)))
        self._start = val

    @property
    def stop(self):
        return self._stop

    @stop.setter
    def stop(self, val):
        if not isinstance(val, int):
            raise TypeError('stop.value must be of type int not{}'.format(type(val)))
        self._stop = val

    @property
    def parent(self):
        return self._parent

    @parent.setter
    def parent(self, val):
        if val is not None and not isinstance(val, Voice):
            raise TypeError('parent.value must be of type Voice not{}'.format(type(val)))
        self._parent = val

    @property
    def lines(self):
        if not self._lines:
            self._lines = self.parent.line_segments[self.start:self.stop]
        return self._lines

    def apply_to_parent(self):
        for line in self.lines:
            line.show = True
            if line != self.lines[0]:
                line.start_mark_line.show = False
            if line == self.lines[-1]:
                line.end_mark_line.show = True


class Voice(AbstractVoice, Named):
    def __init__(self, segments=None, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._segments = []
        self.segments = segments
        self.hide_all()

    def hide_all(self):
        for line_segment in self.line_segments:
            line_segment.show = False

    @property
    def segments(self):
        return self._segments

    @segments.setter
    def segments(self, val):
        if val is None:
            self._segments = []
        else:
            if not isinstance(val, list):
                raise TypeError('segments.value must be of type list not{}'.format(type(val)))
            self._segments = []
            for v in val:
                self.add_voice_segment(*v)

    def add_voice_segment(self, start, end):
        vs = VoiceSegment(start, end, parent=self)
        self._segments.append(vs)
        vs.apply_to_parent()
        return vs



from pathlib import Path

from musurgia.pdf.line import HorizontalRuler
from musurgia.pdf.pdf import Pdf
from musurgia.unittest import TestCase

path = Path(__file__)


class TestRuler(TestCase):
    def setUp(self) -> None:
        self.pdf = Pdf()

    def test_h_ruler(self):
        r = HorizontalRuler(length=50)
        with self.file_path(path, 'h_ruler', 'pdf') as pdf_path:
            self.pdf.translate_page_margins()
            r.draw(self.pdf)
            self.pdf.write(pdf_path)

    def test_h_ruler_A4(self):
        with self.file_path(path, 'h_ruler_A4', 'pdf') as pdf_path:
            self.pdf.translate_page_margins()
            self.pdf.draw_ruler(mode='h')
            self.pdf.write(pdf_path)

    def test_both_rulers_A4(self):
        with self.file_path(path, 'both_rulers_A4', 'pdf') as pdf_path:
            self.pdf.translate_page_margins()
            self.pdf.draw_ruler(mode='h')
            self.pdf.draw_ruler(mode='v')
            self.pdf.write(pdf_path)



import os

from musicscore.musictree.treescoretimewise import TreeScoreTimewise

from musurgia.unittest import TestCase
from musurgia.fractaltree.fractalmusic import FractalMusic

path = os.path.abspath(__file__).split('.')[0]


class Test(TestCase):
    def setUp(self) -> None:
        self.fm = FractalMusic(tempo=60, tree_permutation_order=[3, 1, 2], proportions=[1, 2, 3], quarter_duration=20)
        self.score = TreeScoreTimewise()

    def test_1(self):
        self.fm.add_layer()
        self.fm.add_layer()
        self.fm.add_layer()
        sf = self.fm.get_simple_format(layer=1)
        v = sf.to_stream_voice(1)
        v.add_to_score(self.score)
        xml_path = path + '_test_1.xml'
        self.score.write(xml_path)
        self.assertCompareFiles(xml_path)



from math import ceil
from pathlib import Path

from musurgia.pdf.line import HorizontalRuler, HorizontalSegmentedLine
from musurgia.pdf.newdrawobject import ClippingArea
from musurgia.pdf.pdf import Pdf
from musurgia.pdf.rowcolumn import DrawObjectColumn
from musurgia.pdf.text import PageText
from musurgia.unittest import TestCase, create_test_path

path = Path(__file__)


class TestClip(TestCase):
    def setUp(self) -> None:
        self.pdf = Pdf()

    def test_line(self):
        pdf_path = create_test_path(path, 'line.pdf')
        self.pdf.rect(0, 0, 50, 50)
        self.pdf.clip_rect(0, 0, 50, 50)
        self.pdf.line(10, 20, 100, 100)
        self.pdf.write(pdf_path)
        self.assertCompareFiles(pdf_path)

    def test_line_break(self):
        def draw_with_clip(index):
            with self.pdf.saved_state():
                self.pdf.clip_rect(-1, -5, 196, 50)
                self.pdf.translate(index * -190, 0)
                ruler.draw(self.pdf)

        ruler = HorizontalRuler(length=800, unit=10)
        with self.file_path(path, 'line_break', 'pdf') as pdf_path:
            self.pdf.translate(10, 10)
            number_of_rows = int(ceil(ruler.length / 190))
            for index in range(number_of_rows):
                if index != 0:
                    self.pdf.translate(0, 30)
                draw_with_clip(index)
            self.pdf.write(pdf_path)

    def test_column_line_break(self):
        def draw_with_clip(index):
            with self.pdf.saved_state():
                self.pdf.clip_rect(-1, -5, 196, 50)
                self.pdf.translate(index * -190, 0)
                c.draw(self.pdf)

        c = DrawObjectColumn()
        c.bottom_margin = 10
        c.add_draw_object(HorizontalRuler(length=800))
        c.add_draw_object(HorizontalSegmentedLine(lengths=400 * [2]))
        with self.file_path(path, 'column_line_break', 'pdf') as pdf_path:
            self.pdf.translate_page_margins()
            number_of_rows = int(ceil(c.get_width() / 190))
            for index in range(number_of_rows):
                if index != 0:
                    self.pdf.translate(0, c.get_height())
                draw_with_clip(index)
            self.pdf.write(pdf_path)

    def test_column_page_break(self):
        self.pdf.t_margin = 15

        def _prepare_page():
            self.pdf.translate_page_margins()
            self.pdf.draw_ruler('v')
            self.pdf.draw_ruler('h')
            self.pdf.translate(clip_area_left_margin, clip_area_top_margin)

        def _add_page():
            self.pdf.add_page()
            _prepare_page()

        def draw_with_clip(index):
            with self.pdf.saved_state():
                self.pdf.clip_rect(-1, -5, clip_area_width + 1.14, clip_area_height)
                self.pdf.translate(index * -clip_area_width, 0)
                c.draw(self.pdf)

        c = DrawObjectColumn()
        c.bottom_margin = 60
        c.add_draw_object(HorizontalRuler(length=1200, bottom_margin=5))
        c.add_draw_object(HorizontalSegmentedLine(lengths=600 * [2]))

        clip_area_left_margin = 10
        clip_area_top_margin = 10
        clip_area_width = self.pdf.w - self.pdf.l_margin - self.pdf.r_margin - clip_area_left_margin
        clip_area_height = c.get_height()

        with self.file_path(path, 'column_page_break', 'pdf') as pdf_path:
            _prepare_page()
            number_of_rows = int(ceil(c.get_width() / clip_area_width))
            for index in range(number_of_rows):
                if index != 0:
                    self.pdf.translate(0, c.get_height())
                if self.pdf.absolute_y > self.pdf.h - self.pdf.b_margin:
                    _add_page()
                draw_with_clip(index)
            self.pdf.write(pdf_path)

    def test_with_clipping_area(self):
        c = DrawObjectColumn()
        c.bottom_margin = 60
        c.add_draw_object(HorizontalRuler(length=1200, bottom_margin=5))
        c.add_draw_object(HorizontalSegmentedLine(lengths=600 * [2]))
        ca = ClippingArea(self.pdf, draw_object=c)
        self.pdf.translate_page_margins()
        ca.draw()
        with self.file_path(path, 'with_clipping_area', 'pdf') as pdf_path:
            self.pdf.write(pdf_path)

    def test_clipped_draw(self):
        c = DrawObjectColumn()
        c.bottom_margin = 60
        c.add_draw_object(HorizontalRuler(length=1200, bottom_margin=5))
        c.add_draw_object(HorizontalSegmentedLine(lengths=600 * [2]))
        self.pdf.translate_page_margins()
        title = PageText('A very nice title', v_position='center', font_weight='bold', font_size=12, top_margin=10)
        title.draw(self.pdf)
        self.pdf.translate(0, title.get_height())
        c.clipped_draw(self.pdf)

        for page in self.pdf.pages:
            self.pdf.page = page
            self.pdf.reset_position()
            self.pdf.translate_page_margins()
            self.pdf.draw_ruler('h')
            self.pdf.draw_ruler('v')
        with self.file_path(path, 'clipped_draw', 'pdf') as pdf_path:
            self.pdf.write(pdf_path)



from pathlib import Path

from musurgia.pdf.labeled import Labeled
from musurgia.pdf.line import VerticalSegmentedLine
from musurgia.pdf.masterslave import PositionMaster
from musurgia.pdf.pdf import Pdf
from musurgia.pdf.text import TextLabel
from musurgia.unittest import TestCase

path = Path(__file__)


class LabeledVerticalLine(VerticalSegmentedLine, Labeled):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def draw(self, pdf):
        self.draw_above_text_labels(pdf)
        self.draw_left_text_labels(pdf)
        super().draw(pdf)

    # def get_slave_position(self, slave, position):
    #     if isinstance(slave, TextLabel):
    #         return 0
    #     else:
    #         return super().get_slave_position(slave, position)


class DummyPositionMaster(PositionMaster):
    def get_slave_position(self, slave, position):
        if position == 'x':
            return 0
        elif position == 'y':
            return 0


class TestTextLabel(TestCase):
    def setUp(self) -> None:
        self.pdf = Pdf()

    def test_draw(self):
        t = TextLabel(master=DummyPositionMaster(), name='t1', text='Fox is going to be dead.')

        with self.file_path(path, 'draw', 'pdf') as pdf_path:
            self.pdf.translate_page_margins()
            self.pdf.draw_ruler('h')
            self.pdf.draw_ruler('v')
            self.pdf.translate(10, 10)
            t.draw(self.pdf)
            self.pdf.write(pdf_path)

    def test_draw_multiple(self):
        t1 = TextLabel(master=DummyPositionMaster(), name='t1', text='Fox is going to be dead.')
        t1.top_margin = -5
        t2 = TextLabel(master=DummyPositionMaster(), name='t2', text='What should we do??')
        t3 = TextLabel(master=DummyPositionMaster(), name='t3', text='What should we do??')
        with self.file_path(path, 'draw_multiple', 'pdf') as pdf_path:
            self.pdf.translate_page_margins()
            self.pdf.draw_ruler('h')
            self.pdf.draw_ruler('v')
            self.pdf.translate(10, 10)
            t1.draw(self.pdf)
            self.pdf.translate(0, t1.get_height())
            t2.draw(self.pdf)
            self.pdf.translate(0, t2.get_height())
            t3.draw(self.pdf)
            self.pdf.translate(0, t3.get_height())
            self.pdf.write(pdf_path)

    def test_left_labels(self):
        unit = 1
        length = 40

        lvl = LabeledVerticalLine(lengths=int(length / unit) * [unit])
        for index, segment in enumerate(lvl.segments):
            if index % 10 == 0:
                segment.start_mark_line.length = 3
            else:
                segment.start_mark_line.length = 1.5

        lvl.add_label('first one one', placement='left', font_size=8)
        lvl.add_label('second two two', placement='left')
        lvl.add_label('third three three', placement='left', font_size=12)
        lvl.add_label('fourth four four', placement='left')
        lvl.add_label('fifth five five', placement='left')
        lvl.add_label('sixth six six', placement='left')
        lvl.add_label('seventh seven seven', placement='left')
        with self.file_path(path, 'left_labels', 'pdf') as pdf_path:
            self.pdf.translate_page_margins()
            self.pdf.draw_ruler('h')
            self.pdf.draw_ruler('v')
            self.pdf.translate(30, 10)
            lvl.draw(self.pdf)
            self.pdf.write(pdf_path)



from fractions import Fraction
from unittest import TestCase

from musurgia.fractaltree.fractalmusic import FractalMusic


def get_infos(fm):
    return [fm.tempo, fm.quarter_duration, fm.duration]


def print_infos(fm):
    print('tempo={}; quarter_duration={}; duration={};'.format(
        *get_infos(fm)))


class Test(TestCase):
    def setUp(self) -> None:
        self.fm = FractalMusic(tree_permutation_order=[3, 1, 2], proportions=[1, 2, 3], duration=20)

    def test_1(self):
        # print(self.fm.tempo)
        # print(self.fm.quarter_duration)
        # print(self.fm.duration)
        # print_infos(self.fm)
        self.fm.tempo = 60
        result = [60, Fraction(20, 1), Fraction(20, 1)]
        self.assertEqual(result, get_infos(self.fm))

    def test_2(self):
        self.fm.tempo = 60
        self.fm.duration = 10
        # print_infos(self.fm)
        result = [60, Fraction(10, 1), Fraction(10, 1)]
        # print(get_infos(self.fm))
        self.assertEqual(result, get_infos(self.fm))

    def test_3(self):
        self.fm.duration = 10
        self.fm.tempo = 72
        # print_infos(self.fm)
        result = [72, Fraction(12, 1), Fraction(10, 1), ]
        # print(get_infos(self.fm))
        self.assertEqual(result, get_infos(self.fm))

    def test_5(self):
        self.fm.duration = 10
        self.fm.tempo = 72
        self.fm.quarter_duration = 30
        # print_infos(self.fm)
        result = [72, 30, 25]
        # print(get_infos(self.fm))
        self.assertEqual(result, get_infos(self.fm))

    def test_6(self):
        self.fm.duration = 10
        self.fm.tempo = 72
        self.fm.quarter_duration = 40
        self.fm.duration = 20
        # print_infos(self.fm)
        result = [72, 24, 20]
        # print(get_infos(self.fm))
        self.assertEqual(result, get_infos(self.fm))



from unittest import TestCase

from musurgia.fractaltree.fractaltree import FractalTree, FractalTreeException, SetValueFirst


class Test(TestCase):
    def test_1(self):
        ft = FractalTree()
        ft.value = 50
        ft.add_layer()
        self.assertEqual(sum([leaf.value for leaf in ft.get_leaves()]), ft.value)

    def test_2(self):
        ft = FractalTree()
        ft.value = 50
        ft.add_layer()
        with self.assertRaises(FractalTreeException):
            ft.value = 100

    def test_3(self):
        ft = FractalTree()
        with self.assertRaises(SetValueFirst):
            ft.add_layer()
